#include "ESP8266.h"

#include <Wire.h>

//SODAQ Mbili libraries
#include <Sodaq_BMP085.h>
#include <Sodaq_SHT2x.h>
#include <Sodaq_DS3231.h>

//#define SSID        "5 Swiss Hotel - resto"
//#define PASSWORD    "5swiss321"
//#define SSID        "KIST Wifi"
//#define PASSWORD    "NAN"
#define SSID "tkwi-fi"
#define PASSWORD   "wifitk@2015"
#define HOST_NAME   "api.thingspeak.com"
#define HOST_PORT   (80)

//The delay between the sensor readings 
#define SEND_DELAY 60000

//Add your apikey (THINGSPEAK_KEY) from your channel
// GET /update?key=[THINGSPEAK_KEY]&field1=[data 1]&field2=[data 2]...;
String GET = "GET /update?key=WS1NTJF5SLZNRWN2";

//These constants are used for reading the battery voltage
#define ADC_AREF 3.3
#define BATVOLTPIN A6
#define BATVOLT_R1 4.7
#define BATVOLT_R2 10

//TPH BMP sensor
Sodaq_BMP085 bmp;

ESP8266 wifi(Serial1);

//Data header
#define DATA_HEADER "TempSHT21, TempBMP, PressureBMP, HumiditySHT21, Voltage"
void setup(void)
{
    Serial.begin(9600);
    Serial.print("setup begin\r\n");
    
    //Initialise sensors
    setupSensors();
   
    //Setup Wifi connection
    setupWifi();
  
    //Echo the data header to the serial connection
    Serial.println(DATA_HEADER);
    Serial.print("setup end\r\n");
}
 
void loop(void)
{
  //Create TCP connection
  createTCP();
  
  //Read sensor values
  String Temp1=String(SHT2x.GetTemperature());
  String Temp2=String(bmp.readTemperature());
  String Press=String(bmp.readPressure() / 100);
  String Hum=String(SHT2x.GetHumidity());
  
  //Read the voltage
  int mv = getRealBatteryVoltage() * 1000.0;
  String Smv=String(mv);
  
  Serial.print(Temp1);Serial.print(",");
  Serial.print(Temp2);Serial.print(",");
  Serial.print(Press);Serial.print(",");
  Serial.print(Hum);Serial.print(",");
  Serial.println(Smv);
  
  //post to TS
  updateTS( Temp1, Temp2, Press, Hum, Smv );
  delay(SEND_DELAY); //
    
}     

//----- update the  Thingspeak string with 3 values
void updateTS( String T1, String T2 , String P, String H, String V)
{
  //create GET message
  String cmd = GET + "&field1=" + T1 +"&field2="+ T2 + "&field3=" + P +"&field4=" + H +"&field5=" + V+"\r\n";
   
   char buf[150]; //make buffer large enough for 7 digits
   cmd.toCharArray(buf, sizeof(buf));
    
    wifi.send((const uint8_t*)buf, strlen(buf));
    
    uint8_t buffer[128] = {0};
    uint32_t len = wifi.recv(buffer, sizeof(buffer), 10000);
    if (len > 0) {
        Serial.print("Received:[");
        for(uint32_t i = 0; i < len; i++) {
            Serial.print((char)buffer[i]);
        }
        Serial.print("]\r\n");
    }
    
    if (wifi.releaseTCP()) {
        Serial.print("release tcp ok\r\n");
    } else {
        Serial.print("release tcp err\r\n");
    }
    delay(20000);
}



void createTCP(){
    uint8_t buffer[128] = {0};
    
    if (wifi.createTCP(HOST_NAME, HOST_PORT)) {
        Serial.print("create tcp ok\r\n");
    } else {
        Serial.print("create tcp err\r\n");
    }
}

void setupSensors()
{
  //Initialise the wire protocol for the TPH sensors
  Wire.begin();

  //Initialise the TPH BMP sensor
  bmp.begin();

  //Initialise the DS3231 RTC
  rtc.begin();
}


     
void setupWifi(){      
    Serial.print("FW Version:");
    Serial.println(wifi.getVersion().c_str());
      
    if (wifi.setOprToStationSoftAP()) {
        Serial.print("to station + softap ok\r\n");
    } else {
        Serial.print("to station + softap err\r\n");
    }
 
    if (wifi.joinAP(SSID, PASSWORD)) {
        Serial.print("Join AP success\r\n");
        Serial.print("IP:");
        Serial.println( wifi.getLocalIP().c_str());       
    } else {
        Serial.print("Join AP failure\r\n");
    }
    
    if (wifi.disableMUX()) {
        Serial.print("single ok\r\n");
    } else {
        Serial.print("single err\r\n");
    }
  }
  

float getRealBatteryVoltage()
{
  uint16_t batteryVoltage = analogRead(BATVOLTPIN);
  return (ADC_AREF / 1023.0) * (BATVOLT_R1 + BATVOLT_R2) / BATVOLT_R2 * batteryVoltage;
}
